<?php

namespace App\Factory;

use App\Exception;

class ConsumerFactory {
    
    private $connection;
    
    public function __construct(\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function getCityCode($state, $city) {
        $sql = "
            SELECT CDMUNICIPIO 
              FROM MUNICIPIO
             WHERE UPPER(SGESTADO) = :SGESTADO
               AND CDPAIS   = :CDPAIS
               AND NVL((SELECT CDMUNICIPIO
                        FROM MUNICIPIO
                       WHERE UPPER(NMMUNICIPIO) = :NMMUNICIPIO
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
                         AND ROWNUM = 1),
                     (SELECT CDMUNICIPIO
                        FROM MUNICIPIO
                       WHERE UPPER(NMMUNICIPIO) LIKE '%'||:NMMUNICIPIO||'%'
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
                         AND ROWNUM = 1)) = CDMUNICIPIO 
        ";
            
        $params = [
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => strtoupper($state),
            ":NMMUNICIPIO" => strtoupper($city)
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result)) return null;
        
        return $result[0]["CDMUNICIPIO"];
    }
    
    public function getNeighborhoodCode ($consumerDetails) {
        $sql = "
            SELECT CDBAIRRO
            
              FROM BAIRRO
              
            WHERE CDPAIS          = :CDPAIS
            
              AND SGESTADO        = :SGESTADO
              AND CDMUNICIPIO     = :CDMUNICIPIO
              AND UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)";
        
        $params = array(
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => $consumerDetails["SGESTADO"],
            ":CDMUNICIPIO" => $consumerDetails["city"],
            ":NMBAIRRO"    => $consumerDetails["neighborhood"]
        );
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result))
            return null;
        
        return $result[0]["CDBAIRRO"];
    }
    
    public function registerConsumer ($consumerDetails) {
        $sql = "
            INSERT INTO CONSUMIDOR (
                CDCLIENTE, CDCONSUMIDOR, NMCONSUMIDOR, IDSITCONSUMI,
                CDCCUSCLIE, IDCONSUMIDOR, IDATUCONSUMI, IDTPVENDACONS,
                IDIMPCPFCUPOM, IDCADCONFLIBCON, IDPERCONSPRODEX, IDTPSELMANHA,
                IDTPSEALMOCO, IDTPSELTARDE, IDCRACHAMESTRE, CDPAIS,
                SGESTADO, CDMUNICIPIO, CDTIPOCONS, CDIDCONSUMID,
                DSENDECONS, NMBAIRCONS, NRCEPCONS, NRTELECONS,
                DSEMAILCONS, NMRESPCONS, DSREFERENDECONS, DSINFOCONS
            )
            VALUES (
                :CDCLIENTE, :CDCONSUMIDOR, :NMCONSUMIDOR, :IDSITCONSUMI,
                :CDCCUSCLIE, :IDCONSUMIDOR, :IDATUCONSUMI, :IDTPVENDACONS,
                :IDIMPCPFCUPOM, :IDCADCONFLIBCON, :IDPERCONSPRODEX, :IDTPSELMANHA,
                :IDTPSEALMOCO, :IDTPSELTARDE, :IDCRACHAMESTRE, :CDPAIS,
                :SGESTADO, :CDMUNICIPIO, :CDTIPOCONS, :CDIDCONSUMID,
                :DSENDECONS, :NMBAIRCONS, :NRCEPCONS, :NRTELECONS,
                :DSEMAILCONS, :NMRESPCONS, :DSREFERENDECONS, :DSINFOCONS
            )";
        
        $params = [
            ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
            ":NMCONSUMIDOR"    => substr($consumerDetails["NMCONSUMIDOR"], 0, 50),
            ":CDTIPOCONS"      => $consumerDetails["CDTIPOCONS"],
            ":CDCCUSCLIE"      => $consumerDetails["CDCCUSCLIE"],
            ":IDSITCONSUMI"    => "1",
            ":IDCONSUMIDOR"    => "C",
            ":IDATUCONSUMI"    => "S",
            ":IDTPVENDACONS"   => "7",
            ":IDIMPCPFCUPOM"   => "S",
            ":IDCADCONFLIBCON" => "N",
            ":IDPERCONSPRODEX" => "S",
            ":IDTPSELMANHA"    => "N", 
            ":IDTPSEALMOCO"    => "N", 
            ":IDTPSELTARDE"    => "N",
            ":IDCRACHAMESTRE"  => "N",
            ":CDPAIS"          => "",
            ":SGESTADO"        => "",
            ":CDMUNICIPIO"     => "",
            ":CDIDCONSUMID"    => "",
            ":DSENDECONS"      => "",
            ":NMBAIRCONS"      => "",
            ":NRCEPCONS"       => "",
            ":NRTELECONS"      => "",
            ":DSEMAILCONS"     => "",
            ":NMRESPCONS"      => "",
            ":DSREFERENDECONS" => "",
            ":DSINFOCONS"      => ""
        ];
        
        $this->connection->executeQuery($sql, $params);
        
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR
    			   
              FROM CONSUMIDOR

             WHERE CDCONSUMIDOR = :CDCONSUMIDOR
               AND CDCLIENTE    = :CDCLIENTE
		";
		    
		$params = [
            ":CDCONSUMIDOR" => $consumerDetails['CDCONSUMIDOR'],
            ":CDCLIENTE"    => $consumerDetails['CDCLIENTE']
        ];
        
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    private function sanatizeConsumerPhone($number) {
        return preg_replace("/[^0-9]/", "", $number);
    }
    
    public function getStandardConsumer($cdFilial) {
        $sql = "
            SELECT CDCLIENTE, NVL(CDCONSUMIDOR, CDCONSUMIDPADRAO) AS CDCONSUMIDOR, CDTIPOCONS, CDCCUSCLIE
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL
        ";

        $params = [
            ":CDFILIAL" => $cdFilial
        ];

        $response = $this->connection->fetchAssoc($sql, $params);

        if(empty($response)) {
            throw new Exception\ParamsException("
                Não foi possivel buscar a parametrização padrão da filial {$cdFilial}.<br>
                Verifique a seção 'Parametrização do Delivery', no manual da integração.");
        }

        if(empty($response['CDCLIENTE'])) {
            throw new Exception\ParamsException("
                Não existe um Cliente padrão atrelado a filial {$cdFilial}.<br>
                Para atrelar o Cliente padrão desta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");
        }

        if(empty($response['CDTIPOCONS'])) {
            throw new Exception\ParamsException("
                Não existe um tipo de consumidor padrão associado a filial {$cdFilial}.<br>
                Para associar o tipo de consumidor para esta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");
        }

        if(empty($response['CDCCUSCLIE'])) {
            throw new Exception\ParamsException("
                Não existe um centro de custo associado a filial {$cdFilial}.<br>
                Para associar o centro de custo para esta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");
        }

        if(empty($response['CDCONSUMIDOR'])) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Consumidor padrão (Filial: $cdFilial) não encontrado, realizando registro automatico...\n");

            $consumer = [
                "CDCLIENTE"       => $response['CDCLIENTE'],
                "CDCONSUMIDOR"    => ("SDE" . $this->util->newCode("CONSUMIDOR" . $response['CDCLIENTE'], 18)),
                "NMCONSUMIDOR"    => substr("CONSUMIDOR PADRAO", 0, 50),
                "CDTIPOCONS"      => $response["CDTIPOCONS"],
                "CDCCUSCLIE"      => $response["CDCCUSCLIE"]
            ];
            
            $stdConsumer = $this->registerConsumer($consumer);
            
            if(empty($stdConsumer) || empty($stdConsumer['CDCLIENTE']) || empty($stdConsumer['CDCONSUMIDOR'])) {
                throw new Exception\ParamsException("Não foi possivel registrar o consumidor padrão da filial $cdFilial, realize a parametrização conforme o manual da integração.");
            }
            
            $this->updateStandardConsumer($cdFilial, $stdConsumer['CDCLIENTE'], $stdConsumer['CDCONSUMIDOR']);
            
            $response['CDCLIENTE']    = $stdConsumer['CDCLIENTE'];
            $response['CDCONSUMIDOR'] = $stdConsumer['CDCONSUMIDOR'];
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Consumidor padrão (Filial: $cdFilial) registrado (CDCLIENTE: {$stdConsumer['CDCLIENTE']}- CDCONSUMIDOR: {$stdConsumer['CDCONSUMIDOR']})\n");
        }

        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
    			   NMCONSUMIDOR, NRCPFRESPCON,
    			   CDTIPOCONS, NRRGCONSUMID,
    			   DSEMAILCONS, DTNASCCONS,
    			   NRCEPCONS, DSEXIBENDE, 
    			   'PRINCIPAL' AS CDENDECONS, CDPAIS, 
    			   DSENDECONS, NRENDECONS, 
    			   DSCOMPLENDECONS, NRTELECONS, 
    			   NRCELULARCONS, DSREFERENDECONS,
                   CDCONSFACEBOOK, DSENDEIMGCONS,
                   CDTOKENCONSFB, DTEXPTOKENFB,
                   NRLATCONS, NRLONGCONS, 
                   IDCONSEXT
		    
              FROM CONSUMIDOR

             WHERE CDCLIENTE    = :CDCLIENTE
               AND CDCONSUMIDOR = :CDCONSUMIDOR
		";
		    
		$params = [
            ":CDCLIENTE"    => $response['CDCLIENTE'],
            ":CDCONSUMIDOR" => $response['CDCONSUMIDOR']
        ];
        
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function updateStandardConsumer($cdFilial, $cdCliente, $cdConsumidor) {
        $sql = "
            UPDATE PARAVEND
               
               SET CDCONSUMIDPADRAO = :CDCONSUMIDOR
             
             WHERE CDCLIENTE        = :CDCLIENTE
               AND CDFILIAL         = :CDFILIAL
        ";
        
        $params = [
            ":CDCLIENTE"       => $cdCliente,
            ":CDCONSUMIDOR"    => $cdConsumidor,
            ":CDFILIAL"        => $cdFilial,
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
}
